@@ -30,6 +30,8 @@
 #include <gio/gio.h>
 #include <gvfsdaemondbus.h>
 #include <gvfsdaemonprotocol.h>
+#include "gdaemonfile.h"
+#include "metatree.h"
 
 #define OBJ_PATH_PREFIX "/org/gtk/vfs/client/enumerator/"
 
@@ -51,10 +53,12 @@ struct _GDaemonFileEnumerator
 
   /* For async ops, also protected by infos lock */
   int async_requested_files;
-  GCancellable *async_cancel;
   gulong cancelled_tag;
   guint timeout_tag;
   GSimpleAsyncResult *async_res;
+
+  GFileAttributeMatcher *matcher;
+  MetaTree *metadata_tree;
 };
 
 G_DEFINE_TYPE (GDaemonFileEnumerator, g_daemon_file_enumerator, G_TYPE_FILE_ENUMERATOR)
@@ -107,6 +111,10 @@ g_daemon_file_enumerator_finalize (GObject *object)
 
   free_info_list (daemon->infos);
 
+  g_file_attribute_matcher_unref (daemon->matcher);
+  if (daemon->metadata_tree)
+    meta_tree_unref (daemon->metadata_tree);
+
   if (daemon->sync_connection)
     dbus_connection_unref (daemon->sync_connection);
   
@@ -145,23 +153,129 @@ g_daemon_file_enumerator_init (GDaemonFileEnumerator *daemon)
 }
 
 GDaemonFileEnumerator *
-g_daemon_file_enumerator_new (GFile *file)
+g_daemon_file_enumerator_new (GFile *file,
+			      const char *attributes)
 {
   GDaemonFileEnumerator *daemon;
+  char *treename;
 
   daemon = g_object_new (G_TYPE_DAEMON_FILE_ENUMERATOR,
                          "container", file,
                          NULL);
-  
+
+  daemon->matcher = g_file_attribute_matcher_new (attributes);
+  if (g_file_attribute_matcher_enumerate_namespace (daemon->matcher, "metadata") ||
+      g_file_attribute_matcher_enumerate_next (daemon->matcher) != NULL)
+    {
+      treename = g_mount_spec_to_string (G_DAEMON_FILE (file)->mount_spec);
+      daemon->metadata_tree = meta_tree_lookup_by_name (treename, FALSE);
+      g_free (treename);
+    }
+
   return daemon;
 }
 
+static gboolean
+enumerate_keys_callback (const char *key,
+			 MetaKeyType type,
+			 gpointer value,
+			 gpointer user_data)
+{
+  GFileInfo  *info = user_data;
+  char *attr;
+
+  attr = g_strconcat ("metadata::", key, NULL);
+
+  if (type == META_KEY_TYPE_STRING)
+    g_file_info_set_attribute_string (info, attr, (char *)value);
+  else
+    g_file_info_set_attribute_stringv (info, attr, (char **)value);
+
+  g_free (attr);
+
+  return TRUE;
+}
+
+static void
+add_metadata (GFileInfo *info,
+	      GDaemonFileEnumerator *daemon)
+{
+  GFile *container;
+  const char *name;
+  char *path;
+
+  if (!daemon->metadata_tree)
+    return;
+
+  name = g_file_info_get_name (info);
+  container = g_file_enumerator_get_container (G_FILE_ENUMERATOR (daemon));
+  path = g_build_filename (G_DAEMON_FILE (container)->path, name, NULL);
+
+  g_file_info_set_attribute_mask (info, daemon->matcher);
+  meta_tree_enumerate_keys (daemon->metadata_tree, path,
+			    enumerate_keys_callback, info);
+  g_file_info_unset_attribute_mask (info);
+
+  g_free (path);
+}
+
+static GCancellable *
+simple_async_result_get_cancellable (GSimpleAsyncResult *res)
+{
+  return g_object_get_data (G_OBJECT (res), "file-enumerator-cancellable");
+}
+
+static void
+simple_async_result_set_cancellable (GSimpleAsyncResult *res,
+                                     GCancellable       *cancellable)
+{
+  if (!cancellable)
+    return;
+
+  g_object_set_data_full (G_OBJECT (res),
+                          "file-enumerator-cancellable",
+                          g_object_ref (cancellable),
+                          g_object_unref);
+}
+
 /* Called with infos lock held */
 static void
 trigger_async_done (GDaemonFileEnumerator *daemon, gboolean ok)
 {
   GList *rest, *l;
 
+  if (daemon->cancelled_tag != 0)
+    {
+      GCancellable *cancellable = simple_async_result_get_cancellable (daemon->async_res);
+
+      /* If ok, we're a normal callback on the main thread,
+	 ensure protection against a thread cancelling and
+	 running the callback again.
+	 If !ok then we're in a callback which may be
+	 from another thread, but we're guaranteed that
+	 cancellation will only happen once. However
+	 we can't use g_cancellable_disconnect, as this
+	 deadlocks if we call it from within the signal
+	 handler.
+      */
+      if (ok)
+	g_cancellable_disconnect (cancellable,
+				  daemon->cancelled_tag);
+      else
+	g_signal_handler_disconnect (cancellable,
+				     daemon->cancelled_tag);
+    }
+
+  /* cancelled signal handler won't execute after this */
+
+  /* TODO: There is actually a small race here, where
+     trigger_async_done could happen twice if the operation
+     succeeds for some other reason and cancellation happens
+     on some other thread right before the above call.
+     However, this can only happen if cancellation happens
+     outside the main thread which is kinda unusual for async
+     ops. However, it would be nice to handle this too. */
+
   if (ok)
     {
       l = daemon->infos;
@@ -173,7 +287,9 @@ trigger_async_done (GDaemonFileEnumerator *daemon, gboolean ok)
 	  rest->prev = NULL;
 	}
       daemon->infos = rest;
-      
+
+      g_list_foreach (l, (GFunc)add_metadata, daemon);
+
       g_simple_async_result_set_op_res_gpointer (daemon->async_res,
 						 l,
 						 (GDestroyNotify)free_info_list);
@@ -181,10 +297,6 @@ trigger_async_done (GDaemonFileEnumerator *daemon, gboolean ok)
 
   g_simple_async_result_complete_in_idle (daemon->async_res);
   
-  if (daemon->cancelled_tag != 0)
-    g_signal_handler_disconnect (daemon->async_cancel,
-				 daemon->cancelled_tag);
-  daemon->async_cancel = 0;
   daemon->cancelled_tag = 0;
 
   if (daemon->timeout_tag != 0)
@@ -290,7 +402,10 @@ g_daemon_file_enumerator_next_file (GFileEnumerator *enumerator,
 	  done = TRUE;
 	  info = daemon->infos->data;
 	  if (info)
-	    g_assert (G_IS_FILE_INFO (info));
+	    {
+	      g_assert (G_IS_FILE_INFO (info));
+	      add_metadata (G_FILE_INFO (info), daemon);
+	    }
 	  daemon->infos = g_list_delete_link (daemon->infos, daemon->infos);
 	}
       else if (daemon->done)
@@ -349,14 +464,19 @@ async_cancelled (GCancellable *cancellable,
 				   G_IO_ERROR,
 				   G_IO_ERROR_CANCELLED,
 				   _("Operation was cancelled"));
+  G_LOCK (infos);
   trigger_async_done (daemon, FALSE);
+  G_UNLOCK (infos);
 }
 
 static gboolean
 async_timeout (gpointer data)
 {
   GDaemonFileEnumerator *daemon = G_DAEMON_FILE_ENUMERATOR (data);
+
+  G_LOCK (infos);
   trigger_async_done (daemon, TRUE);
+  G_UNLOCK (infos);
   return FALSE;
 }
 
@@ -387,12 +507,12 @@ g_daemon_file_enumerator_next_files_async (GFileEnumerator     *enumerator,
     }
   
   G_LOCK (infos);
-  daemon->async_cancel = cancellable;
   daemon->cancelled_tag = 0;
   daemon->timeout_tag = 0;
   daemon->async_requested_files = num_files;
   daemon->async_res = g_simple_async_result_new (G_OBJECT (enumerator), callback, user_data,
 						 g_daemon_file_enumerator_next_files_async);
+  simple_async_result_set_cancellable (daemon->async_res, cancellable);
 
   /* Maybe we already have enough info to fulfill the requeust already */
   if (daemon->done ||
@@ -400,10 +520,12 @@ g_daemon_file_enumerator_next_files_async (GFileEnumerator     *enumerator,
     trigger_async_done (daemon, TRUE);
   else
     {
-      if (cancellable)
-	daemon->cancelled_tag = g_signal_connect (cancellable, "cancelled", (GCallback)async_cancelled, daemon);
       daemon->timeout_tag = g_timeout_add (G_VFS_DBUS_TIMEOUT_MSECS,
 					   async_timeout, daemon);
+      if (cancellable)
+	daemon->cancelled_tag =
+	  g_cancellable_connect (cancellable, (GCallback)async_cancelled,
+				 daemon, NULL);
     }
   
   G_UNLOCK (infos);
@@ -411,12 +533,24 @@ g_daemon_file_enumerator_next_files_async (GFileEnumerator     *enumerator,
 
 static GList *
 g_daemon_file_enumerator_next_files_finish (GFileEnumerator  *enumerator,
-					    GAsyncResult     *result,
+					    GAsyncResult     *res,
 					    GError          **error)
 {
+  GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (res);
+  GCancellable *cancellable;
   GList *l;
 
-  l = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (result));
+  cancellable = simple_async_result_get_cancellable (result);
+  if (g_cancellable_is_cancelled (cancellable))
+    {
+      g_set_error (error,
+                   G_IO_ERROR,
+                   G_IO_ERROR_CANCELLED,
+                   "%s", _("Operation was cancelled"));
+      return NULL;
+    }
+
+  l = g_simple_async_result_get_op_res_gpointer (result);
   g_list_foreach (l, (GFunc)g_object_ref, NULL);
   return g_list_copy (l);
 }
@@ -445,6 +579,7 @@ g_daemon_file_enumerator_close_async (GFileEnumerator      *enumerator,
 
   res = g_simple_async_result_new (G_OBJECT (enumerator), callback, user_data,
 				   g_daemon_file_enumerator_close_async);
+  simple_async_result_set_cancellable (res, cancellable);
   g_simple_async_result_complete_in_idle (res);
   g_object_unref (res);
 }
@@ -454,5 +589,17 @@ g_daemon_file_enumerator_close_finish (GFileEnumerator      *enumerator,
 				       GAsyncResult         *result,
 				       GError              **error)
 {
+  GCancellable *cancellable;
+  
+  cancellable = simple_async_result_get_cancellable (G_SIMPLE_ASYNC_RESULT (result));
+  if (g_cancellable_is_cancelled (cancellable))
+    {
+      g_set_error (error,
+                   G_IO_ERROR,
+                   G_IO_ERROR_CANCELLED,
+                   "%s", _("Operation was cancelled"));
+      return FALSE;
+    }
+
   return TRUE;
 }
