@@ -36,7 +36,7 @@
 #include "gvfsdaemondbus.h"
 #include <gvfsdaemonprotocol.h>
 #include <gdaemonvfs.h>
-#include "gdbusutils.h"
+#include "gvfsdbusutils.h"
 #include "gsysutils.h"
 
 /* Extra vfs-specific data for DBusConnections */
@@ -804,6 +804,7 @@ _g_vfs_daemon_call_sync (DBusMessage *message,
 
       reply = dbus_pending_call_steal_reply (pending);
       dbus_pending_call_unref (pending);
+      g_cancellable_release_fd (cancellable);
     }
   else
     {
