@@ -31,7 +31,7 @@
 #include <gio/gio.h>
 #include "gmountoperationdbus.h"
 #include "gvfsdaemonprotocol.h"
-#include "gdbusutils.h"
+#include "gvfsdbusutils.h"
 #include <glib/gi18n-lib.h>
 
 typedef struct 
@@ -51,6 +51,10 @@ static void              mount_op_ask_password        (GMountOperationDBus *op_d
 						       DBusMessage         *message);
 static void              mount_op_ask_question        (GMountOperationDBus *op_dbus,
 						       DBusMessage         *message);
+static void              mount_op_show_processes      (GMountOperationDBus *op_dbus,
+						       DBusMessage         *message);
+static void              mount_op_aborted             (GMountOperationDBus *op_dbus,
+						       DBusMessage         *message);
 
 static void
 g_mount_operation_dbus_free (GMountOperationDBus *op_dbus)
@@ -131,6 +135,14 @@ mount_op_message_function (DBusConnection  *connection,
 					G_VFS_DBUS_MOUNT_OPERATION_INTERFACE,
 					G_VFS_DBUS_MOUNT_OPERATION_OP_ASK_QUESTION))
     mount_op_ask_question (op_dbus, message);
+  else if (dbus_message_is_method_call (message,
+					G_VFS_DBUS_MOUNT_OPERATION_INTERFACE,
+					G_VFS_DBUS_MOUNT_OPERATION_OP_SHOW_PROCESSES))
+    mount_op_show_processes (op_dbus, message);
+  else if (dbus_message_is_method_call (message,
+					G_VFS_DBUS_MOUNT_OPERATION_INTERFACE,
+					G_VFS_DBUS_MOUNT_OPERATION_OP_ABORTED))
+    mount_op_aborted (op_dbus, message);
   else
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
@@ -156,18 +168,20 @@ mount_op_send_reply (GMountOperationDBus *op_dbus,
 
 static void
 ask_password_reply (GMountOperation *op,
-		    gboolean abort,
+		    GMountOperationResult result,
 		    gpointer data)
 {
   DBusMessage *reply = data;
   const char *username, *password, *domain;
   dbus_bool_t anonymous;
   guint32 password_save;
-  dbus_bool_t handled = TRUE;
-  dbus_bool_t abort_dbus = abort;
+  dbus_bool_t handled, abort_dbus;
   GMountOperationDBus *op_dbus;
 
   op_dbus = g_object_get_data (G_OBJECT (op), "dbus-op");
+
+  handled = (result != G_MOUNT_OPERATION_UNHANDLED);
+  abort_dbus = (result == G_MOUNT_OPERATION_ABORTED);
   
   password = g_mount_operation_get_password (op);
   if (password == NULL)
@@ -199,12 +213,10 @@ mount_op_ask_password (GMountOperationDBus *op_dbus,
 		       DBusMessage *message)
 {
   const char *message_string, *default_user, *default_domain;
-  dbus_bool_t handled = FALSE;
   guint32 flags;
   DBusMessageIter iter;
   DBusMessage *reply;
   DBusError error;
-  gboolean res;
 
   reply = NULL;
 
@@ -225,6 +237,7 @@ mount_op_ask_password (GMountOperationDBus *op_dbus,
       if (!dbus_connection_send (op_dbus->connection, reply, NULL))
 	_g_dbus_oom ();
       dbus_message_unref (reply);
+      dbus_error_free (&error);
       return;
     }
   
@@ -238,31 +251,24 @@ mount_op_ask_password (GMountOperationDBus *op_dbus,
 			 message_string,
 			 default_user,
 			 default_domain,
-			 flags,
-			 &res);
-
-  if (!res)
-    {
-      _g_dbus_message_append_args (reply,
-				   DBUS_TYPE_BOOLEAN, &handled,
-				   0);
-      mount_op_send_reply (op_dbus, reply);
-    }
+			 flags);
 }
 
 static void
 ask_question_reply (GMountOperation *op,
-		    gboolean abort,
+		    GMountOperationResult result,
 		    gpointer data)
 {
   DBusMessage *reply = data;
   guint32 choice;
-  dbus_bool_t handled = TRUE;
-  dbus_bool_t abort_dbus = abort;
+  dbus_bool_t handled, abort_dbus;
   GMountOperationDBus *op_dbus;
 
   op_dbus = g_object_get_data (G_OBJECT (op), "dbus-op");
 
+  handled = (result != G_MOUNT_OPERATION_UNHANDLED);
+  abort_dbus = (result == G_MOUNT_OPERATION_ABORTED);
+  
   choice = g_mount_operation_get_choice (op);
 
   _g_dbus_message_append_args (reply,
@@ -281,10 +287,8 @@ mount_op_ask_question (GMountOperationDBus *op_dbus,
   const char *message_string;
   char **choices;
   int num_choices;
-  dbus_bool_t handled = FALSE;
   DBusMessage *reply;
   DBusError error;
-  gboolean res;
   DBusMessageIter iter;
 
   reply = NULL;
@@ -304,6 +308,7 @@ mount_op_ask_question (GMountOperationDBus *op_dbus,
       if (!dbus_connection_send (op_dbus->connection, reply, NULL))
 	_g_dbus_oom ();
       dbus_message_unref (reply);
+      dbus_error_free (&error);
       return;
     }
   
@@ -315,15 +320,97 @@ mount_op_ask_question (GMountOperationDBus *op_dbus,
 
   g_signal_emit_by_name (op_dbus->op, "ask_question",
 			 message_string,
-			 choices,
-			 &res);
-  if (!res)
-    {
-      _g_dbus_message_append_args (reply,
-				   DBUS_TYPE_BOOLEAN, &handled,
-				   0);
-      mount_op_send_reply (op_dbus, reply);
-    }
+			 choices);
   
   dbus_free_string_array (choices);
 }
+
+static void
+show_processes_reply (GMountOperation *op,
+                      GMountOperationResult result,
+                      gpointer data)
+{
+  DBusMessage *reply = data;
+  guint32 choice;
+  dbus_bool_t handled, abort_dbus;
+  GMountOperationDBus *op_dbus;
+
+  op_dbus = g_object_get_data (G_OBJECT (op), "dbus-op");
+
+  handled = (result != G_MOUNT_OPERATION_UNHANDLED);
+  abort_dbus = (result == G_MOUNT_OPERATION_ABORTED);
+
+  choice = g_mount_operation_get_choice (op);
+
+  _g_dbus_message_append_args (reply,
+			       DBUS_TYPE_BOOLEAN, &handled,
+			       DBUS_TYPE_BOOLEAN, &abort_dbus,
+			       DBUS_TYPE_UINT32, &choice,
+			       0);
+
+  mount_op_send_reply (op_dbus, reply);
+}
+
+static void
+mount_op_show_processes (GMountOperationDBus *op_dbus,
+                         DBusMessage         *message)
+{
+  const char *message_string;
+  char **choices;
+  int num_choices;
+  gint32 **process_pids;
+  int num_process_pids;
+  DBusMessage *reply;
+  DBusError error;
+  DBusMessageIter iter;
+  GArray *processes;
+
+  reply = NULL;
+
+  dbus_message_iter_init (message, &iter);
+  dbus_error_init (&error);
+  if (!_g_dbus_message_iter_get_args (&iter,
+				      &error,
+				      DBUS_TYPE_STRING, &message_string,
+				      DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+				      &choices, &num_choices,
+				      DBUS_TYPE_ARRAY, DBUS_TYPE_INT32,
+				      &process_pids, &num_process_pids,
+				      0))
+    {
+      reply = dbus_message_new_error (message, error.name, error.message);
+      if (reply == NULL)
+	_g_dbus_oom ();
+      if (!dbus_connection_send (op_dbus->connection, reply, NULL))
+	_g_dbus_oom ();
+      dbus_message_unref (reply);
+      dbus_error_free (&error);
+      return;
+    }
+
+  processes = g_array_sized_new (FALSE, FALSE, sizeof (GPid), num_process_pids);
+  g_array_append_vals (processes, process_pids, num_process_pids);
+
+  reply = dbus_message_new_method_return (message);
+  if (reply == NULL)
+    _g_dbus_oom ();
+
+  g_signal_connect (op_dbus->op, "reply", (GCallback)show_processes_reply, reply);
+
+  g_signal_emit_by_name (op_dbus->op, "show_processes",
+			 message_string,
+                         processes,
+			 choices);
+
+  dbus_free_string_array (choices);
+  g_array_unref (processes);
+}
+
+static void
+mount_op_aborted (GMountOperationDBus *op_dbus,
+		  DBusMessage         *message)
+{
+  /* also emit reply to make the all DBus ops return */
+  g_mount_operation_reply (op_dbus->op, G_MOUNT_OPERATION_UNHANDLED);
+  g_signal_emit_by_name (op_dbus->op, "aborted");
+}
