@@ -191,6 +191,94 @@ gvfs_backend_ftp_setup_directory_cache (GVfsBackendFtp *ftp)
   ftp->dir_cache = g_vfs_ftp_dir_cache_new (ftp->dir_funcs);
 }
 
+/* This parses a file according to RFC 959 Appendix II:
+ *
+ * the server should return a line of the form:
+ *   257<space>"<directory-name>"<space><commentary>
+ * That is, the server will tell the user what string to use when
+ * referring to the created  directory.  The directory name can
+ * contain any character; embedded double-quotes should be escaped by
+ * double-quotes (the "quote-doubling" convention).
+ *
+ * Note that we actually accept (as regexp):
+ * ....\s*"<directory-name>".*
+ * so we're quite a bit more lax.
+ *
+ * If parsing this name fails, this function returns NULL.
+ */
+static GVfsFtpFile *
+g_vfs_backend_create_file_from_reply (GVfsBackendFtp *ftp, const char *name)
+{
+  GVfsFtpFile *file;
+  const char *quote;
+  GString *unescaped;
+
+  /* strip leading spaces */
+  while (g_ascii_isspace (*name))
+    name++;
+
+  if (*name++ != '"')
+    {
+      g_debug ("# filename didn't start with a quote\n");
+      return NULL;
+    }
+
+  unescaped = g_string_new (NULL);
+  while ((quote = strchr (name, '"')))
+    {
+      g_string_append_len (unescaped, name, quote-name);
+      name = quote + 2;
+      if (quote[1] == '"')
+        g_string_append_c (unescaped, '"');
+      else
+        break;
+    }
+
+  /* NB: The name variable may point into bad space here */
+  if (quote == NULL)
+    {
+      g_debug ("# filename didn't end with a quote\n");
+      g_string_free (unescaped, TRUE);
+      return NULL;
+    }
+  else if (!g_ascii_isspace (quote[1]))
+    {
+      /* issue a warning here, just so we can guess right from debug logs */
+      g_debug ("# warning: filename didn't contain space after final quote\n");
+    }
+
+  file = g_vfs_ftp_file_new_from_ftp (ftp, unescaped->str);
+
+  g_string_free (unescaped, TRUE);
+  return file;
+}
+
+static void
+gvfs_backend_ftp_determine_default_location (GVfsFtpTask *task)
+{
+  char **reply;
+  GVfsFtpFile *home;
+
+  if (g_vfs_ftp_task_is_in_error (task))
+    return;
+
+  if (!g_vfs_ftp_task_send_and_check (task, 0, NULL, NULL, &reply, "PWD"))
+    {
+      g_vfs_ftp_task_clear_error (task);
+      return;
+    }
+
+  home = g_vfs_backend_create_file_from_reply (task->backend, reply[0] + 4);
+  if (home != NULL)
+    {
+      g_vfs_backend_set_default_location (G_VFS_BACKEND (task->backend),
+                                          g_vfs_ftp_file_get_gvfs_path (home));
+      g_vfs_ftp_file_free (home);
+    }
+
+  g_strfreev (reply);
+}
+
 /*** COMMON FUNCTIONS WITH SPECIAL HANDLING ***/
 
 static gboolean
@@ -313,11 +401,17 @@ do_mount (GVfsBackend *backend,
                 		     port == 21 ? 0 : port,
                 		     &username,
                 		     NULL,
-                		     &password))
+                		     &password) &&
+      username != NULL &&
+      password != NULL)
     {
       anonymous = FALSE;
       goto try_login;
     }
+  g_free (username);
+  g_free (password);
+  username = NULL;
+  password = NULL;
 
   while (TRUE)
     {
@@ -354,7 +448,8 @@ do_mount (GVfsBackend *backend,
                         &password_save) ||
           aborted)
         {
-          g_set_error_literal (&task.error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+          g_set_error_literal (&task.error, G_IO_ERROR,
+                               aborted ? G_IO_ERROR_FAILED_HANDLED : G_IO_ERROR_PERMISSION_DENIED,
                 	       _("Password dialog cancelled"));
           break;
         }
@@ -371,24 +466,24 @@ try_login:
       g_free (ftp->password);
       if (anonymous)
         {
-              ftp->user = g_strdup ("anonymous");
-              ftp->password = g_strdup ("");
+          g_free (username);
+          g_free (password);
+          ftp->user = g_strdup ("anonymous");
+          ftp->password = g_strdup ("");
           if (g_vfs_ftp_task_login (&task, "anonymous", "") != 0)
-            {
-              break;
-            }
+            break;
+          g_free (ftp->user);
+          g_free (ftp->password);
           ftp->user = NULL;
           ftp->password = NULL;
         }
       else
         {
-          ftp->user = username ? g_strdup (username) : g_strdup ("");
-          ftp->password = g_strdup (password);
+          ftp->user = username ? username : g_strdup ("");
+          ftp->password = password;
           if (g_vfs_ftp_task_login (&task, ftp->user, ftp->password) != 0)
             break;
         }
-      g_free (username);
-      g_free (password);
      
       if (break_on_fail ||
           !g_error_matches (task.error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED))
@@ -410,6 +505,7 @@ try_login:
   g_vfs_ftp_task_setup_connection (&task);
   gvfs_backend_ftp_determine_system (&task);
   gvfs_backend_ftp_setup_directory_cache (ftp);
+  gvfs_backend_ftp_determine_default_location (&task);
 
   /* Save the address of the current connection, so that for future connections,
    * we are sure to connect to the same machine.
@@ -421,7 +517,7 @@ try_login:
       ftp->addr = G_SOCKET_CONNECTABLE (g_vfs_ftp_connection_get_address (task.conn, &task.error));
       if (ftp->addr == NULL)
         {
-          g_debug ("# error querying remote address: %s\nUsing original address instead.", task.error->message);
+          g_debug ("# error querying remote address: %s\nUsing original address instead.\n", task.error->message);
           g_vfs_ftp_task_clear_error (&task);
           ftp->addr = g_object_ref (addr);
         }
@@ -905,6 +1001,7 @@ do_query_info (GVfsBackend *backend,
     }
 
   g_vfs_ftp_task_done (&task);
+  g_vfs_ftp_file_free (file);
 }
 
 static void
@@ -925,6 +1022,7 @@ do_enumerate (GVfsBackend *backend,
                                          dir,
                                          TRUE,
                                          query_flags & G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS ? FALSE : TRUE);
+  g_vfs_ftp_file_free (dir);
   if (g_vfs_ftp_task_is_in_error (&task))
     {
       g_assert (list == NULL);
@@ -948,7 +1046,6 @@ do_enumerate (GVfsBackend *backend,
   g_vfs_job_enumerate_done (job);
 
   g_list_free (list);
-  g_vfs_ftp_file_free (dir);
 }
 
 static void
@@ -1165,6 +1262,9 @@ ftp_output_stream_splice (GOutputStream *output,
   GCancellable *current, *timer_cancel;
   gulong cancel_cb_id;
 
+  timer_cancel = NULL;
+  cancel_cb_id = 0;
+
   bytes_copied = 0;
   if (progress_callback)
     {
@@ -1334,6 +1434,16 @@ do_pull (GVfsBackend *         backend,
   src = g_vfs_ftp_file_new_from_gvfs (ftp, source);
   dest = g_file_new_for_path (local_path);
 
+  if (progress_callback)
+    {
+      GFileInfo *info = g_vfs_ftp_dir_cache_lookup_file (ftp->dir_cache, &task, src, TRUE);
+      if (info)
+        {
+          total_size = g_file_info_get_size (info);
+          g_object_unref (info);
+        }
+    }
+
   g_vfs_ftp_task_setup_data_connection (&task);
   g_vfs_ftp_task_send_and_check (&task,
                                  G_VFS_FTP_PASS_100 | G_VFS_FTP_FAIL_200,
@@ -1367,16 +1477,6 @@ do_pull (GVfsBackend *         backend,
       goto out;
     }
 
-  if (progress_callback)
-    {
-      GFileInfo *info = g_vfs_ftp_dir_cache_lookup_file (ftp->dir_cache, &task, src, TRUE);
-      if (info)
-        {
-          total_size = g_file_info_get_size (info);
-          g_object_unref (info);
-        }
-    }
-
   input = g_io_stream_get_input_stream (g_vfs_ftp_connection_get_data_stream (task.conn));
   ftp_output_stream_splice (output,
                             input,
