@@ -51,7 +51,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <errno.h>
-#include <poll.h>
 #include "crc32.h"
 
 #ifdef HAVE_LIBUDEV
@@ -158,6 +157,7 @@ struct _MetaTree {
   int fd;
   char *data;
   gsize len;
+  ino_t inode;
 
   guint32 tag;
   gint64 time_t_base;
@@ -438,6 +438,7 @@ meta_tree_init (MetaTree *tree)
 
 	  dir = g_path_get_dirname (tree->filename);
 	  g_mkdir_with_parents (dir, 0700);
+	  g_free (dir);
 
 	  builder = meta_builder_new ();
 	  retried = TRUE;
@@ -468,6 +469,7 @@ meta_tree_init (MetaTree *tree)
 
   tree->fd = fd;
   tree->len = statbuf.st_size;
+  tree->inode = statbuf.st_ino;
   tree->data = data;
   tree->header = (MetaFileHeader *)data;
 
@@ -613,12 +615,24 @@ meta_tree_unref (MetaTree *tree)
 static gboolean
 meta_tree_needs_rereading (MetaTree *tree)
 {
+  struct stat statbuf;
+
   if (tree->fd == -1)
     return TRUE;
 
   if (tree->header != NULL &&
       GUINT32_FROM_BE (tree->header->rotated) == 0)
     return FALSE; /* Got a valid tree and its not rotated */
+
+  /* Sanity check to avoid infinite loops when a stable file
+     has the rotated bit set to 1 (see gnome bugzilla bug #600057) */
+
+  if (lstat (tree->filename, &statbuf) != 0)
+    return FALSE;
+
+  if (tree->inode == statbuf.st_ino)
+    return FALSE;
+
   return TRUE;
 }
 
@@ -1642,7 +1656,7 @@ meta_tree_lookup_stringv   (MetaTree                         *tree,
       num_strings = GUINT32_FROM_BE (stringv->num_strings);
       res = g_new (char *, num_strings + 1);
       for (i = 0; i < num_strings; i++)
-	res[i] = verify_string (tree, stringv->strings[i]);
+	res[i] = g_strdup (verify_string (tree, stringv->strings[i]));
       res[i] = NULL;
     }
 
@@ -2336,6 +2350,7 @@ static gboolean
 meta_tree_flush_locked (MetaTree *tree)
 {
   MetaBuilder *builder;
+  gboolean res;
 
   builder = meta_builder_new ();
 
@@ -2344,12 +2359,14 @@ meta_tree_flush_locked (MetaTree *tree)
   if (tree->journal)
     apply_journal_to_builder (tree, builder);
 
-  meta_builder_write (builder,
-		      meta_tree_get_filename (tree));
+  res = meta_builder_write (builder,
+			    meta_tree_get_filename (tree));
+  if (res)
+    meta_tree_refresh_locked (tree);
 
-  meta_tree_refresh_locked (tree);
+  meta_builder_free (builder);
 
-  return FALSE;
+  return res;
 }
 
 gboolean
@@ -2971,7 +2988,7 @@ update_mountinfo (void)
   char *contents;
   int res;
   gboolean first;
-  struct pollfd pfd;
+  GPollFD pfd;
 
   first = FALSE;
   if (!mountinfo_initialized)
@@ -2987,9 +3004,9 @@ update_mountinfo (void)
   if (!first)
     {
       pfd.fd = mountinfo_fd;
-      pfd.events = POLLIN | POLLOUT | POLLPRI;
+      pfd.events = G_IO_IN | G_IO_OUT | G_IO_PRI;
       pfd.revents = 0;
-      res = poll (&pfd, 1, 0);
+      res = g_poll (&pfd, 1, 0);
       if (res == 0)
 	return;
     }
@@ -2998,7 +3015,10 @@ update_mountinfo (void)
   contents = read_contents (mountinfo_fd);
   lseek (mountinfo_fd, SEEK_SET, 0);
   if (contents)
-    mountinfo_roots = parse_mountinfo (contents);
+    {
+      mountinfo_roots = parse_mountinfo (contents);
+      g_free (contents);
+    }
 }
 
 static char *
@@ -3083,7 +3103,8 @@ find_mountpoint_for (MetaLookupCache *cache,
   last = g_strdup (file);
   while (1)
     {
-      dir_dev = get_devnum (dir);
+      if (dir)
+	dir_dev = get_devnum (dir);
       if (dir == NULL ||
 	  dev != dir_dev)
 	{
@@ -3164,6 +3185,7 @@ meta_lookup_cache_free (MetaLookupCache *cache)
   g_free (cache->last_parent_expanded);
   g_free (cache->last_parent_mountpoint);
   g_free (cache->last_parent_mountpoint_extra_prefix);
+  g_free (cache->last_device_tree);
   g_free (cache);
 }
 
@@ -3215,6 +3237,7 @@ expand_parents (MetaLookupCache *cache,
       strcmp (cache->last_parent, parent) != 0)
     {
       g_free (cache->last_parent);
+      g_free (cache->last_parent_expanded);
       cache->last_parent = parent;
       cache->last_parent_expanded = expand_all_symlinks (parent, &parent_dev);
       cache->last_parent_dev = parent_dev;
@@ -3228,6 +3251,7 @@ expand_parents (MetaLookupCache *cache,
 
   *parent_dev_out = cache->last_parent_dev;
   basename = g_path_get_basename (path_copy);
+  g_free (path_copy);
   res = g_build_filename (cache->last_parent_expanded, basename, NULL);
   g_free (basename);
 
@@ -3308,6 +3332,7 @@ meta_lookup_cache_lookup_path (MetaLookupCache *cache,
     }
 
  found:
+  g_free (expanded);
   tree = meta_tree_lookup_by_name (treename, for_write);
   if (tree)
     {
