@@ -359,7 +359,7 @@ list_mounts (GList *mounts,
   GMount *mount;
   GVolume *volume;
   char *name, *uuid, *uri;
-  GFile *root;
+  GFile *root, *default_location;
   GIcon *icon;
   char **x_content_types;
   char *type_name;
@@ -394,6 +394,15 @@ list_mounts (GList *mounts,
           if (uuid)
             g_print ("%*suuid=%s\n", indent + 2, "", uuid);
 
+          default_location = g_mount_get_default_location (mount);
+          if (default_location)
+            {
+              char *loc_uri = g_file_get_uri (default_location);
+              g_print ("%*sdefault_location=%s\n", indent + 2, "", loc_uri);
+              g_free (loc_uri);
+              g_object_unref (default_location);
+            }
+
           icon = g_mount_get_icon (mount);
           if (icon)
             {
@@ -893,10 +902,8 @@ monitor_drive_eject_button (GVolumeMonitor *volume_monitor, GDrive *drive)
 static void
 monitor (void)
 {
-  GMainLoop *loop;
   GVolumeMonitor *volume_monitor;
 
-  loop = g_main_loop_new (NULL, FALSE);
   volume_monitor = g_volume_monitor_get ();
 
   g_signal_connect (volume_monitor, "mount-added", (GCallback) monitor_mount_added, NULL);
